// Fill out your copyright notice in the Description page of Project Settings.

#include "MyParticle.h"

// Sets default values
AMyParticle::AMyParticle()
{
 	// Set this actor to call Tick() every frame.  You can turn this off to improve performance if you don't need it.
	PrimaryActorTick.bCanEverTick = true;
	UStaticMeshComponent* smc = CreateDefaultSubobject<UStaticMeshComponent>(TEXT("SMC"));
	RootComponent = MeshComponent = smc;

	UStaticMesh* mesh = ConstructorHelpers::FObjectFinder<UStaticMesh>(TEXT("StaticMesh'/Engine/BasicShapes/Sphere.Sphere'")).Object;
	smc->SetStaticMesh(mesh);
	smc->SetSimulatePhysics(true);
	smc->SetEnableGravity(false);
	smc->SetMassOverrideInKg(NAME_None, 1.0f);
	
	Mat = ConstructorHelpers::FObjectFinder<UMaterial>(TEXT("Material'/Engine/ArtTools/RenderToTexture/Materials/M_DebugSphereBounds_01.M_DebugSphereBounds_01'")).Object;
	//Mat = ConstructorHelpers::FObjectFinder<UMaterial>(TEXT("Material'/LennardJonesPlugin/ParticleMaterial.ParticleMaterial'")).Object;
}

float AMyParticle::ComputeForce(float r)
{	
	//V = 4 * eps * (pow(sig / r, 12) - pow(sig / r, 6)); // potencial field
	U = 4 * eps * ((pow(6 * sig, 6) / pow(r, 7)) - (pow(12 * sig, 12) / pow(r, 13)));

	return U;
}


// Called when the game starts or when spawned
void AMyParticle::BeginPlay()
{
	Super::BeginPlay();

	Material = UMaterialInstanceDynamic::Create(Mat, MeshComponent);
	MeshComponent->SetMaterial(0, Material);
}

// Called every frame
void AMyParticle::Tick(float DeltaTime)
{
	Super::Tick(DeltaTime);
	UWorld* world = this->GetWorld();

	CurrentTime += DeltaTime;
	force = 0.f;
	FVector fce = FVector(0.f, 0.f, 0.f);

	for (TActorIterator<AMyParticle> it(world); it; ++it) {

		if (it->GetUniqueID() == this->GetUniqueID()) {
			//	UE_LOG(LogClass, Warning, TEXT("That's me!"));
			continue;
		}

		auto loc = it->GetActorLocation();
		auto self = GetActorLocation();

		auto res = loc - self;

		float u = ComputeForce(res.Size());
		fce += u * res.GetSafeNormal();
		force += u;

	}

	MeshComponent->AddForce(fce);

	Material->SetScalarParameterValue("Force", FMath::Clamp(FMath::Abs(force) * 1000.f, 0.f, 1.f));
}

